#include <sstream>

#ifdef GTGEN_WITH_SOFA
	#include <fstream>
#endif

#include "cell.h"
#include "toolbox/flowfields.h"
#include "toolbox/bp_lists.h"

//
// look into cell_00_Samplephase.cpp to see full version of the
// template for an atomic cell phase function like this one
//

template <class MV, class PV>
void Cell<MV, PV>::DoG2Phase(const size_t noFrames)
{
	 REPORT("\n   *****   *****   cell ID=" << ID << ", STARTING G2-phase at timePoint=" << 
			  timePoint); 

	// ---------- local working masks ----------
	//local working copy of the cell mask - not needed
	//only nucleus
	i3d::Image3d<MV> maskNucleus;

	maskNucleus.SetResolution(scheduler.sceneRes);
	maskNucleus.SetOffset(scheduler.sceneMasks[timePoint]->GetOffset());
	//but the size of _nucleus_ mask image should contain the whole _cell_
#ifndef GTGEN_WITH_SOFA
	AllocateAndZeroNewMask(maskNucleus, scmCellBPList, noFrames, cellLookDistance);
#else
	AllocateAndZeroNewMask(maskNucleus, scmCellBPList, 2, cellLookDistance);
#endif

	//prepare working copy of cell nucleus with nucleoli
	RenderBPListIntoMask(maskNucleus,(MV)1,
	  scmNucleusBPList,scmNucleusBPCentre,scmNucleusOuterBPNumber);

#ifdef ENABLE_NUCLEOLI
	RenderBPListIntoMask(maskNucleus,(MV)2,
	  scmNucleoli1BPList,scmNucleoli1BPCentre,scmNucleoli1OuterBPNumber);
	RenderBPListIntoMask(maskNucleus,(MV)3,
	  scmNucleoli2BPList,scmNucleoli2BPCentre,scmNucleoli2OuterBPNumber);
	//now in the maskNucleus image: 0 means outside,
	//1 nucleus, 2 and 3 each nucleolus
#endif

#ifdef GTGEN_WITH_SOFA
#ifndef ENABLE_NUCLEOLI
	throw ERROR_REPORT("There are no nucleoli in the nucleus.");
#endif

	//save/export the mask image with the 3 labels
	char fn[100];
	sprintf(fn, "ID%.2u_init_nucleusMask.mhd", this->ID);
	maskNucleus.SaveImage(fn);


	/*
	const float xOff=maskNucleus.GetOffset().x;
	const float yOff=maskNucleus.GetOffset().y;
	const float zOff=maskNucleus.GetOffset().z;
	*/
	//disable subtraction of the image offset
	const float xOff=0.f;
	const float yOff=0.f;
	const float zOff=0.f;

	//export the full list of nucleus boundary/countour points
	sprintf(fn, "ID%.2u_init_nucleusBoundaryPoints.txt", this->ID);
	std::ofstream BPlist(fn);

	std::vector< i3d::Vector3d<float> /**/>::const_iterator BPiter;
	BPiter=scmNucleusBPList.begin();
	while (BPiter != scmNucleusBPList.end())
	{
		BPlist << BPiter->x-xOff << " " << BPiter->y-yOff << " "
				 << BPiter->z-zOff << "\n" ;
		++BPiter;
	}
	BPlist.close();

	//export the "outer" list of nucleus boundary/countour points
	sprintf(fn, "ID%.2u_init_nucleusOuterBoundaryPoints.txt", this->ID);
	BPlist.open(fn,std::ios::out);
	BPiter=scmNucleusBPList.begin();
	size_t counter=0;
	while ( (counter < scmNucleusOuterBPNumber) && (BPiter != scmNucleusBPList.end()) ) 
	{
		BPlist << BPiter->x-xOff << " " << BPiter->y-yOff << " "
				 << BPiter->z-zOff << "\n" ;
		++BPiter;
		++counter;
	}
	BPlist.close();


	//export the full list of the first nucleoli boundary/countour points
	sprintf(fn, "ID%.2u_init_nucleoli1BoundaryPoints.txt", this->ID);
	BPlist.open(fn,std::ios::out);
	BPiter=scmNucleoli1BPList.begin();
	while (BPiter != scmNucleoli1BPList.end())
	{
		BPlist << BPiter->x-xOff << " " << BPiter->y-yOff << " "
				 << BPiter->z-zOff << "\n" ;
		++BPiter;
	}
	BPlist.close();

	//export the "outer" list of the first nucleoli boundary/countour points
	sprintf(fn, "ID%.2u_init_nucleoli1OuterBoundaryPoints.txt", this->ID);
	BPlist.open(fn,std::ios::out);
	BPiter=scmNucleoli1BPList.begin();
	counter=0;
	while ( (counter < scmNucleoli1OuterBPNumber) && (BPiter != scmNucleoli1BPList.end()) )
	{
		BPlist << BPiter->x-xOff << " " << BPiter->y-yOff << " "
				 << BPiter->z-zOff << "\n" ;
		++BPiter;
		++counter;
	}
	BPlist.close();


	//export the full list of the second nucleoli boundary/countour points
	sprintf(fn, "ID%.2u_init_nucleoli2BoundaryPoints.txt", this->ID);
	BPlist.open(fn,std::ios::out);
	BPiter=scmNucleoli2BPList.begin();
	while (BPiter != scmNucleoli2BPList.end())
	{
		BPlist << BPiter->x-xOff << " " << BPiter->y-yOff << " "
				 << BPiter->z-zOff << "\n" ;
		++BPiter;
	}
	BPlist.close();

	//export the "outer" list of the first nucleoli boundary/countour points
	sprintf(fn, "ID%.2u_init_nucleoli2OuterBoundaryPoints.txt", this->ID);
	BPlist.open(fn,std::ios::out);
	BPiter=scmNucleoli2BPList.begin();
	counter=0;
	while ( (counter < scmNucleoli2OuterBPNumber) && (BPiter != scmNucleoli2BPList.end()) )
	{
		BPlist << BPiter->x-xOff << " " << BPiter->y-yOff << " "
				 << BPiter->z-zOff << "\n" ;
		++BPiter;
		++counter;
	}
	BPlist.close();


	//export the list chromatin points
	sprintf(fn, "ID%.2u_init_chromatinPoints.txt", this->ID);
	BPlist.open(fn,std::ios::out);

	std::vector<Molecule>::const_iterator CHRiter=chrDotList.begin();
	while (CHRiter != chrDotList.end())
	{
		BPlist << CHRiter->pos.x-xOff << " " << CHRiter->pos.y-yOff << " "
				 << CHRiter->pos.z-zOff << "\n" ;
		++CHRiter;

	}
	BPlist.close();
	

	//prepare non-rigid deformation plan for export
	scheduler.PnGetMeNonrigidHints(scmNucleusBPList,scmNucleusOuterBPNumber,scmPnHints);
	//note that the previous function makes sure: scmPnHints.GetSizeX() == scmNucleusOuterBPNumber

	sprintf(fn, "ID%.2u_init_nucleusOuterBoundaryPoints_DeformationMagnitudes.ics", this->ID);
	scmPnHints.SaveImage(fn);

	const float *myHintPtr=scmPnHints.GetFirstVoxelAddr();
	for (size_t curTime=0; curTime < scmPnHints.GetSizeY(); ++curTime)
	{
		sprintf(fn, "ID%.2u_T%03lu_nucleusOuterBoundaryPoints_DeformationMagnitudes.txt", this->ID,curTime);
		BPlist.open(fn,std::ios::out);

		float mult=1.f;
		if (curTime == scmPnNextStrongUsage)
		{
			//the code is a copy from Cell::ScmSuggestNonrigidDeformation()
			const float PnStrongInitDelay=10.f; //dalsi strong non-rigid bude prumerne v 10-tem frame
			scmPnNextStrongUsage+=(size_t)std::max(GetRandomGauss(PnStrongInitDelay,PnStrongInitDelay/4.f),1.f);

			DEBUG_REPORT("using stronger hints this time; next such event will be at " << scmPnNextStrongUsage);

			mult=cellNonRigidStrongDeformation/cellNonRigidDeformation;
		}

		for (size_t BPindex=0; BPindex < scmPnHints.GetSizeX(); ++BPindex, ++myHintPtr)
			BPlist << *myHintPtr *mult << "\n";

		BPlist.close();
	}
#endif

	i3d::Image3d<float> *distanceMap = NULL;

	// ---------- helper flow fields ----------
	FlowField<float> FF;
	InitFlowField(maskNucleus,FF);

	// ---------- actions a phase needs to do before the frame generating cycle ----------
	//prepare the activity plan to spread the phase duties over the given number of frames

	//the main cycle
	//for (size_t T=0; T < noFrames; ++T) {
	timePoint+=noFrames; for (size_t T=0; T < 0; ++T) {
		REPORT("\n   *****   *****   cell ID=" << ID << ", doing G2-phase at timePoint=" << timePoint \
		       << " with sync");

		//
		// internals
		//

		//no change of cell shape here

		//just movement of dots:
		//over all chromosomes
		for (size_t i=0; i < chrCount; ++i) 
		{
			ChrMoveWithBrown(i,0.02f,maskNucleus,distanceMap,1);
			//ChrMoveWithGravity(i,0.25f,maskNucleus,distanceMap,1);
		}


		//
		// externals
		//

		//reset the current flow field
		FF.x->GetVoxelData()=0.f;
		FF.y->GetVoxelData()=0.f;
		if (FF.z) FF.z->GetVoxelData()=0.f;

		// bool collapse = (timePoint == scmPnNextStrongUsage) ? true : false;

		ScmUniversalMovement(FF);

		//
		// a footer common to all phase functions:
		//

		//move the boundary points and draw cell into sceneMasks
		MoveBPListAndCentre(scmCellBPList,scmCellBPCentre,FF);
		MoveBPListAndCentre(scmNucleusBPList,scmNucleusBPCentre,FF);

#ifdef ENABLE_NUCLEOLI
		MoveBPListAndCentre(scmNucleoli1BPList,scmNucleoli1BPCentre,FF);
		MoveBPListAndCentre(scmNucleoli2BPList,scmNucleoli2BPCentre,FF);
#endif

		ScmRemoveCellFromMask(*scheduler.sceneMasks[timePoint+1]);
		ScmRenderCellIntoMask(*scheduler.sceneMasks[timePoint+1]
#ifdef GTGEN_WITH_NUCLEIMASKS
			,scheduler.sceneNuclei[timePoint+1]
#endif
		);


		//update also the local masks
		//no need to update cell now, it is not used in the cycle

		//nucleous with nucleoli:
		maskNucleus.Clear();
		RenderBPListIntoMask(maskNucleus,(MV)1,
		  scmNucleusBPList,scmNucleusBPCentre,scmNucleusOuterBPNumber);

#ifdef ENABLE_NUCLEOLI
		RenderBPListIntoMask(maskNucleus,(MV)2,
		  scmNucleoli1BPList,scmNucleoli1BPCentre,scmNucleoli1OuterBPNumber);
		RenderBPListIntoMask(maskNucleus,(MV)3,
		  scmNucleoli2BPList,scmNucleoli2BPCentre,scmNucleoli2OuterBPNumber);
#endif

		if (distanceMap)
		{
			 delete distanceMap; 
			 distanceMap = NULL; // force recomputing distance map
		}

		//update dots in the masks
		ChrMoveByFlow(FF, maskNucleus, distanceMap,(MV)1);

		ChrUpdateStats();

		// if strong deformation happened, call the repair function
		// if (collapse)
		// call 'repair' function each 3rd frame
		if ((this->timePoint % RESTART_PERIOD) == 0)
		{ 
			 // smazme - begin
			 /*
			 i3d::Image3d<PV> kesmazani;
			 kesmazani.CopyMetaData(*scheduler.scenePhantoms[timePoint+1]);
			 char fn[50];
			 sprintf(fn, "ID%.2u_T%.3lu_predKorekci.ics", this->ID, this->timePoint);
			 DEBUG_REPORT("Saving: " << fn);
			 ChrRenderIntoPhantoms(kesmazani);
			 kesmazani.SaveImage(fn);
			 */
			 // smazme - end

			 /*
			 // select appropriate points 
			 std::vector<bool> overcrowded(chrCount);
			 for (size_t i=0; i<chrCount; i++)
			 {
				  overcrowded[i] = chrDensities[i] > 170.0f; //was: 160.0f;
			 }

			 ChrRepairDistrLocally(maskNucleus, overcrowded, distanceMap, MV(1));
			 */
			 ChrRepairDistrWeighted(maskNucleus, distanceMap, MV(1));
			 ChrUpdateStats();
		}

#ifdef GTGEN_DEBUG
		ChrReportStats();
		ChrReportClosests();
#endif

		//here, one may possibly add UnevenDistributionCorrection...
		//problematic chromosomes are likely those with chrDensities[]>25

		//draw the phantom image
		ChrRenderIntoPhantoms(*scheduler.scenePhantoms[timePoint+1]);
#ifdef GTGEN_WITH_TERRITORIES
		ChrRenderIntoTerritories(*scheduler.sceneTerritories[timePoint+1]);
#endif

		++timePoint;
	}

	//cleaning here if necessary
	
	if (distanceMap)
		 delete distanceMap;

	if (centrosomeA) delete centrosomeA;
	if (centrosomeB) delete centrosomeB;

	// create centrosomes
	centrosomeA = new Dot;
	centrosomeB = new Dot;

	//render cell mask for the GenerateCentrosomes() function
	//into a new fresh only-this purpose image
	i3d::Image3d<bool> maskCell;
	RenderBPListIntoNewMask(maskCell,true,
		scmCellBPList,scmCellBPCentre,scmCellOuterBPNumber,0, cellLookDistance);

	GenerateCentrosomes(maskCell, centrosomeA->pos, centrosomeB->pos);

	//since cell is doing non-rigid, boundary points move appart
	//we rather re-sample them again
	scmCellOuterBPNumber=
	  ResetBoundaryPoints(scmCellBPList,scmCellBPCentre,*scheduler.sceneMasks[timePoint],ID);
	//note that here we must use current timePoint, the one already finished

	//changing boundary points, we need to re-init the hints image
	scheduler.PnGetMeNonrigidHints(scmCellBPList,scmCellOuterBPNumber,scmPnHints);

	scmNucleusOuterBPNumber=
	  ResetBoundaryPoints(scmNucleusBPList,scmNucleusBPCentre,maskNucleus);

#ifdef ENABLE_NUCLEOLI
	scmNucleoli1OuterBPNumber=
	  ResetBoundaryPoints(scmNucleoli1BPList,scmNucleoli1BPCentre,maskNucleus,(MV)2);
	scmNucleoli2OuterBPNumber=
	  ResetBoundaryPoints(scmNucleoli2BPList,scmNucleoli2BPCentre,maskNucleus,(MV)3);
#endif
}


//----------------------------------------------------------------------
// explicit instantiations to force explicit build of the functions 
// in this file because otherwise the linker couldn't find them
//----------------------------------------------------------------------
template class Cell<i3d::GRAY8,  i3d::GRAY16>;
template class Cell<i3d::GRAY16, i3d::GRAY16>;
